import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import "Controller.js" as Controller
import IconTheme 1.0

Rectangle {
    id : previewWindow
    width : 100
    height : 150
    color : "transparent"

    /** windowType is the type of PreviewWindow,
    * the options are : none, app, desktop, creation.
    * Default to none.
    */
    property string windowType : "none"

    /** stateList is the list of state,
    * the options are : default, hover, drag, receive.
    * Default to default.
    */
    property var stateList : ["default", "hover", "drag", "receive"];
    state : "default"
    property string wid : "";
    property int previewAreaWidth : 100
    property int previewAreaHeight : 150
    property real posX : 0
    property real posY : 0
    property double closeButtonSize : 24
    property bool closeButtonVisble : false
    property double titleOpacity : 1
    property var iconVisble : true
    property var controller : null
    property real borderWidth : 0
    property var dragAxis : Drag.XAndYAxis
    property var captionText  : ""
    property var captionColor : Qt.rgba(255,255,255,1)
    property bool winowDragActive : previewMouseArea.drag.active
    property bool nowDragActive : false
    property bool onTop: false
    property ListModel premodel : null
    Drag.active : previewMouseArea.drag.active
    Drag.source : previewWindow
    Drag.hotSpot.x : previewWindow.previewAreaWidth
    Drag.hotSpot.y : previewWindow.previewAreaHeight

    onWinowDragActiveChanged : {
        if(winowDragActive !== nowDragActive){
            nowDragActive = winowDragActive
        }
    }

    NumberAnimation on x{
        id : movePreX
        duration : 500
        running : false
    }

    NumberAnimation on y{
        id : movePreY
        duration : 500
        running : false
    }

    function setStatus(status) {
        if(stateList.indexOf(status) < 0)
        {
            return;
        }
        if (status === state) {
            return
        } else {
            state = status;
        }
    }

    function setCaption(titleText) {
        caption.text = titleText;
    }

    function setId(wId) {
        wid = wId;
        /*此处加载关闭按钮的id的原因如下
        * 1.此类被虚拟桌面和预览窗口，同时复用；
        * 2.父子组件的onCompleted{},子类先触发，然后才是父类
        * 3.如果在Component.onCompleted{}中，给closebutton赋id时，只有model.id不为空，其他均为空，原因为2导致
        * 4.虚拟桌面没有model的概念，会在3处赋值的时候，报warning
        * 故以上原因，在此处赋id
        */
        closeButton.setId(wId)
    }

    function setDefaultStatus() {
        setStatus("default")
    }

    function setHoverStatus() {
        setStatus("hover")
    }

    function setDragStatus() {
        setStatus("drag")
    }

    function setReceiveStatus() {
        setStatus("receive")
    }

    function returnOriginal(x,y)
    {
        movePreX.to =0
        movePreY.to =0
        movePreX.from =x
        movePreY.from =y
        movePreX.stop()
        movePreY.stop()
        movePreX.start()
        movePreY.start()
    }

    // 应用程序预览窗口的缩放动画暂时
    // 由视图模块在响应拖拽事件时主动
    // 触发。因此，该函数暂时实现为空
    function onEnterDropArea() {

    }

    // 应用程序预览窗口的缩放动画暂时
    // 由视图模块在响应拖拽事件时主动
    // 触发。因此，该函数暂时实现为空
    function onExitDropArea() {

    }

    function zoomIn() {
        scaleAnimation.running = false
        scaleAnimation.stop();
        scaleAnimation.from = 1;
        scaleAnimation.to = 0.25;
        scaleAnimation.start();
    }

    function zoomOut() {
        scaleAnimation.running = false
        scaleAnimation.stop();
        scaleAnimation.from = 0.5;
        scaleAnimation.to = 1;
        scaleAnimation.start();
    }

    NumberAnimation on scale{
        id : scaleAnimation
        duration : 500
        from : 0
        to : 0
        running : false
    }

    MouseArea{
        id : previewMouseArea
        anchors.fill : parent
        hoverEnabled : true
        preventStealing : true
        drag.target : previewWindow
        drag.axis : dragAxis

        onEntered : {
            controller.setHoverStatus();
        }

        onExited : {
            if(!closeButton.closeButtonContainMouse)
            {
                controller.setDefaultStatus();
            }
        }

        onClicked : {
            controller.onWindowClicked(wid);
        }

        onReleased : {
            previewWindow.Drag.drop();
        }
    }


    ColumnLayout {
        spacing : 2
        anchors.centerIn : parent
        Layout.fillHeight : true
        width : parent.width

        RowLayout {
            id : captionRow
            spacing : 8
            Layout.fillHeight : true
            opacity : titleOpacity
            anchors.rightMargin: 6

            Rectangle {
                id : iconItem
                height : 24
                width : 24
                color : "transparent"
                visible : iconVisble
                IconTheme{
                    anchors.fill : parent
                    icon : multitaskViewModel.getWindowIcon(wid)
                }
            }

            Label {
                id : caption
                verticalAlignment: Qt.AlignVCenter
                elide : Text.ElideRight
                Layout.fillWidth : true
                height : 24
                text : captionText
                font.pixelSize: multitaskViewModel.getUKUIFontSize()
                color: captionColor
            }

            CloseButton {
                id : closeButton
                wid: previewWindow.wid

                states :  [
                    State {
                        name : "default"
                        PropertyChanges { target : closeButton; closeButtonColor : closeButtonDefaultColor }
                    },
                    State {
                        name : "hover"
                        PropertyChanges { target : closeButton; closeButtonColor : closeButtonHoverColor }
                    },
                    State {
                        name : "click"
                        PropertyChanges { target : closeButton; closeButtonColor : closeButtonClickColor }
                    }
                ]
            }
        }//RowLayout

        Rectangle {
            id : previewArea
            width : previewAreaWidth
            height : previewAreaHeight
            color : "transparent"
            border.color : "#FFFFFF"
            border.width : borderWidth
            radius : multitaskViewManager.useOpenGL() ? 12 : 0
        }

        states : State {
            when : previewMouseArea.drag.active && windowType != "creation"
            ParentChange { target : previewWindow; parent : root }
            PropertyChanges { target : previewWindow; onTop : true }
            PropertyChanges { target : previewWindow; borderWidth : 4 }
            PropertyChanges { target : previewWindow; titleOpacity : 0 }
        }
    }//culumnLayout
}
